----------------------------------------------------------------------------
--  ser_rxtx.vhd
--	Example SerDes 
--	Version 1.0
--
--  Copyright (C) 2015 H.Poetzl
--
--	This program is free software: you can redistribute it and/or
--	modify it under the terms of the GNU General Public License
--	as published by the Free Software Foundation, either version
--	2 of the License, or (at your option) any later version.
--
----------------------------------------------------------------------------



library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.numeric_std.ALL;


entity ser_rxtx is
    port (
	clk : in std_logic;
	rx : in std_logic;
	tx : out std_logic;
	--
  	rx_data : out std_logic_vector (7 downto 0);
	rx_full : in std_logic;
	rx_wen : out std_logic;
	--
  	tx_data : in std_logic_vector (7 downto 0);
	tx_empty : in std_logic;
	tx_ren : out std_logic
    );

end entity ser_rxtx;


architecture RTL of ser_rxtx is
begin

    --------------------------------------------------------------------
    -- Deserializer
    --------------------------------------------------------------------

    rx_proc : process (clk)
	variable cnt_v : natural range 0 to 7 := 7;
	variable data_v : std_logic_vector(7 downto 0) := x"55";
    begin
	rx_wen <= '0';
	if rising_edge(clk) then
	    data_v(cnt_v) := rx;

	    if cnt_v = 0 then
		if not rx_full then
		    rx_data <= data_v;
		    rx_wen <= '1';
		end if;
		cnt_v := 7;
	    else
		cnt_v := cnt_v - 1;
	    end if;
	end if;
    end process;

    --------------------------------------------------------------------
    -- Serializer
    --------------------------------------------------------------------

    tx_proc : process (clk)
	variable cnt_v : natural range 0 to 7 := 7;
	variable data_v : std_logic_vector(7 downto 0) := x"AA";
    begin
	tx_ren <= '0';
	if rising_edge(clk) then
	    tx <= data_v(cnt_v);

	    if cnt_v = 0 then
		if not tx_empty then
		    data_v := tx_data;
		    tx_ren <= '1';
		end if;
		cnt_v := 7;
	    else
		cnt_v := cnt_v - 1;
	    end if;
	end if;
    end process;


end RTL;
